@@ -583,7 +583,7 @@ class LearningShapelets(ClassifierMixin, TransformerMixin,
             [X[:, :, di].reshape((n_ts, sz, 1)) for di in range(self.d_)],
             batch_size=self.batch_size, verbose=self.verbose
         )
-        return locations.astype(numpy.int)
+        return locations.astype(int)
 
     def _check_series_length(self, X):
         """Ensures that time series in X matches the following requirements:
@@ -888,129 +888,3 @@ class LearningShapelets(ClassifierMixin, TransformerMixin,
 
 
 ShapeletModel = LearningShapelets
-
-
-class SerializableShapeletModel(LearningShapelets):
-    """Serializable variant of the Learning Time-Series Shapelets model.
-
-
-    Learning Time-Series Shapelets was originally presented in [1]_.
-    
-    .. deprecated:: 0.4
-             `SerializableShapeletModel` is deprecated in version 0.4 and
-            will be removed in 0.6. Use `LearningShapelets` instead, which is
-            now fully serializable and clonable.
-
-    Parameters
-    ----------
-    n_shapelets_per_size: dict (default: None)
-        Dictionary giving, for each shapelet size (key),
-        the number of such shapelets to be trained (value)
-    max_iter: int (default: 10,000)
-        Number of training epochs.
-
-        .. versionchanged:: 0.3
-            default value for max_iter is set to 10,000 instead of 100
-
-    batch_size: int (default:256)
-        Batch size to be used.
-
-    verbose: {0, 1, 2} (default: 0)
-        `keras` verbose level.
-    learning_rate: float (default: 0.01)
-        Learning rate to be used for the SGD optimizer.
-    weight_regularizer: float or None (default: 0.)
-        Strength of the L2 regularizer to use for training the classification
-        (softmax) layer. If 0, no regularization is performed.
-    shapelet_length: float (default 0.15)
-        The length of the shapelets, expressed as a fraction of the ts length
-    total_lengths: int (default 3)
-        The number of different shapelet lengths. Will extract shapelets of
-        length i * shapelet_length for i in [1, total_lengths]
-    random_state : int or None, optional (default: None)
-        The seed of the pseudo random number generator to use when shuffling
-        the data.  If int, random_state is the seed used by the random number
-        generator; If None, the random number generator is the RandomState
-        instance used by `np.random`.
-
-    Attributes
-    ----------
-    shapelets_ : numpy.ndarray of objects, each object being a time series
-        Set of time-series shapelets.
-
-    shapelets_as_time_series_ : numpy.ndarray of shape (n_shapelets, sz_shp, \
-            d) where `sz_shp` is the maximum of all  shapelet sizes
-        Set of time-series shapelets formatted as a ``tslearn`` time series
-        dataset.
-
-    transformer_model_ : keras.Model
-        Transforms an input dataset of timeseries into distances to the
-        learned shapelets.
-
-    locator_model_ : keras.Model
-        Returns the indices where each of the shapelets can be found (minimal
-        distance) within each of the timeseries of the input dataset.
-
-    model_ : keras.Model
-        Directly predicts the class probabilities for the input timeseries.
-
-    Notes
-    -----
-        This implementation requires a dataset of equal-sized time series.
-
-    Examples
-    --------
-    >>> from tslearn.generators import random_walk_blobs
-    >>> X, y = random_walk_blobs(n_ts_per_blob=10, sz=16, d=2, n_blobs=3)
-    >>> clf = SerializableShapeletModel(n_shapelets_per_size={4: 5},
-    ...                                 max_iter=1, verbose=0,
-    ...                                 learning_rate=0.01)
-    >>> _ = clf.fit(X, y)
-    >>> clf.shapelets_.shape[0]
-    5
-    >>> clf.shapelets_[0].shape
-    (4, 2)
-    >>> clf.predict(X).shape
-    (30,)
-    >>> clf.predict_proba(X).shape
-    (30, 3)
-    >>> clf.transform(X).shape
-    (30, 5)
-
-    References
-    ----------
-    .. [1] J. Grabocka et al. Learning Time-Series Shapelets. SIGKDD 2014.
-    """
-    def __init__(self, n_shapelets_per_size=None,
-                 max_iter=10000,
-                 batch_size=256,
-                 verbose=0,
-                 learning_rate=0.01,
-                 weight_regularizer=0.,
-                 shapelet_length=0.3,
-                 total_lengths=3,
-                 random_state=None):
-        super().__init__(n_shapelets_per_size=n_shapelets_per_size,
-                             max_iter=max_iter,
-                             batch_size=batch_size,
-                             verbose=verbose,
-                             weight_regularizer=weight_regularizer,
-                             shapelet_length=shapelet_length,
-                             total_lengths=total_lengths,
-                             random_state=random_state)
-        self.learning_rate = learning_rate
-        warnings.warn(
-            "`SerializableShapeletModel` is deprecated in version 0.4 and "
-            "will be removed in 0.6. Use `LearningShapelets` instead, which is"
-            " now fully serializable and clonable.",
-            DeprecationWarning, stacklevel=2)
-
-    def _set_model_layers(self, X, ts_sz, d, n_classes):
-        super()._set_model_layers(X=X,
-                                  ts_sz=ts_sz,
-                                  d=d,
-                                  n_classes=n_classes)
-        K.set_value(self.model_.optimizer.lr, self.learning_rate)
-
-    def set_params(self, **params):
-        return super().set_params(**params)
