@@ -234,18 +234,6 @@ class TimeSeriesSVC(TimeSeriesSVMMixin, ClassifierMixin,
                       'it is non-trivial to access the underlying libsvm')
         return 1
 
-    @deprecated('The use of '
-                '`support_vectors_time_series_` is deprecated in '
-                'tslearn v0.4 and will be removed in v0.6. Use '
-                '`support_vectors_` property instead.')
-    def support_vectors_time_series_(self, X=None):
-        warnings.warn('The use of '
-                      '`support_vectors_time_series_` is deprecated in '
-                      'tslearn v0.4 and will be removed in v0.6. Use '
-                      '`support_vectors_` property instead.')
-        check_is_fitted(self, '_X_fit')
-        return self._X_fit[self.svm_estimator_.support_]
-
     @property
     def support_vectors_(self):
         check_is_fitted(self, '_X_fit')
@@ -470,7 +458,7 @@ class TimeSeriesSVR(TimeSeriesSVMMixin, RegressorMixin,
     >>> from tslearn.generators import random_walk_blobs
     >>> X, y = random_walk_blobs(n_ts_per_blob=10, sz=64, d=2, n_blobs=2)
     >>> import numpy
-    >>> y = y.astype(numpy.float) + numpy.random.randn(20) * .1
+    >>> y = y.astype(float) + numpy.random.randn(20) * .1
     >>> reg = TimeSeriesSVR(kernel="gak", gamma="auto")
     >>> reg.fit(X, y).predict(X).shape
     (20,)
@@ -509,18 +497,6 @@ class TimeSeriesSVR(TimeSeriesSVMMixin, RegressorMixin,
                       'it is non-trivial to access the underlying libsvm')
         return 1
 
-    @deprecated('The use of '
-                '`support_vectors_time_series_` is deprecated in '
-                'tslearn v0.4 and will be removed in v0.6. Use '
-                '`support_vectors_` property instead.')
-    def support_vectors_time_series_(self, X=None):
-        warnings.warn('The use of '
-                      '`support_vectors_time_series_` is deprecated in '
-                      'tslearn v0.4 and will be removed in v0.6. Use '
-                      '`support_vectors_` property instead.')
-        check_is_fitted(self, '_X_fit')
-        return self._X_fit[self.svm_estimator_.support_]
-
     @property
     def support_vectors_(self):
         check_is_fitted(self, '_X_fit')
